/**********************************************************************
*  2008 Microchip Technology Inc.
*
* FileName:        main.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       dsPIC33Fxxxx
* Compiler:        MPLAB C30 v3.02 or higher
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Incorporated ("Microchip") retains all ownership and 
* intellectual property rights in the code accompanying this message and in all 
* derivatives hereto.  You may use this code, and any derivatives created by 
* any person or entity by or on your behalf, exclusively with Microchip's
* proprietary products.  Your acceptance and/or use of this code constitutes 
* agreement to the terms and conditions of this notice.
*
* CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS".  NO 
* WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
* TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIP'S 
* PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
*
* YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE, WHETHER 
* IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), 
* STRICT LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, SPECIAL, 
* PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, FOR COST OR EXPENSE OF 
* ANY KIND WHATSOEVER RELATED TO THE CODE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN 
* ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT 
* ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO 
* THIS CODE, SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO 
* HAVE THIS CODE DEVELOPED.
*
* You agree that you are solely responsible for testing the code and 
* determining its suitability.  Microchip has no obligation to modify, test, 
* certify, or support the code.
*
* ADDITIONAL NOTES:
* Code Tested on:
* 16-Bit 28-Pin Starter board with dsPIC33FJ128GP802 device 
**********************************************************************/

#include "p33fxxxx.h"
#include "dsp.h"
#include "..\h\adcdacDrv.h"
#include "..\h\table.h"
#include <delay.h>

//Macros for Configuration Fuse Registers:
//Invoke macros to set up  device configuration fuse registers.
//The fuses will select the oscillator source, power-up timers, watch-dog
//timers etc. The macros are defined within the device
//header files. The configuration fuse registers reside in Flash memory.



// Internal FRC Oscillator
_FOSCSEL(FNOSC_FRC);							// FRC Oscillator
_FOSC(FCKSM_CSECMD & OSCIOFNC_ON  & POSCMD_NONE); 
												// Clock Switching is enabled and Fail Safe Clock Monitor is disabled
												// OSC2 Pin Function: OSC2 is Clock Output
												// Primary Oscillator Mode: Disabled

_FWDT(FWDTEN_OFF);              				// Watchdog Timer Enabled/disabled by user software
												// (LPRC can be disabled by clearing SWDTEN bit in RCON register

/* Constant Definitions */
#define BLOCK_LENGTH    256             			/*We will filter a block of 256 samples in this example*/

/* extern definitions for the imported *.s files from dsPIC Filter Design */
extern FIRStruct filterFilter; /*Contains filter structures for FIR-BPF*/

/* Filter Operation Vectors */
fractional FilterOut[BLOCK_LENGTH] ;    			/*Output array where filtered output will*/
                                        			/*be stored */


int main (void)
{
	// Configure Oscillator to operate the device at 40MIPS
	// Fosc= Fin*M/(N1*N2), Fcy=Fosc/2
	// Fosc= 7.37M*43/(2*2)=79.22Mhz for ~40MIPS input clock
	PLLFBD=41;									// M=43
	CLKDIVbits.PLLPOST=0;						// N1=2
	CLKDIVbits.PLLPRE=0;						// N2=2
	OSCTUN=0;									// Tune FRC oscillator, if FRC is used

	// Disable Watch Dog Timer
	RCONbits.SWDTEN=0;

	// Clock switch to incorporate PLL
	__builtin_write_OSCCONH(0x01);				// Initiate Clock Switch to
												// FRC with PLL (NOSC=0b001)
	__builtin_write_OSCCONL(0x01);				// Start clock switching
	while (OSCCONbits.COSC != 0b001);			// Wait for Clock switch to occur

	// Wait for PLL to lock
	while(OSCCONbits.LOCK!=1) {};
	

   	initAdc();              					// Initialize the A/D converter to convert Channel 4
	initDac(); 									// Initialize the D/A converter 
	initDma0();									// Initialize the DMA controller to buffer ADC data in conversion order
	initTmr3();									// Initialize the Timer to generate sampling event for ADC

	extern fractional BufferA[NUMSAMP];			// Ping pong buffer A
	extern fractional BufferB[NUMSAMP];			// Ping pong buffer B
	extern unsigned int DmaBuffer;				// DMA flag
	extern int flag;							// Flag
 	int i;


	// Let's get this party started...
	int bitArray[25];
 	initS2P();
	clearDisplay();
	testAllLights();
//	fullDisplayFlash();

	/*Initialize the filter state variables (delay line) prior to calling */
    /*the FIR() routine the very first time */
    FIRDelayInit(&filterFilter);

 	
    while (1)               					// Loop Endlessly - Execution is interrupt driven
    { 			 
		if(flag) 
		{      

			for(i = 0; i < NUMSAMP; i++)
			{
				while(DAC1STATbits.REMPTY != 1);// Wait for D/A conversion
				if(DmaBuffer == 0){
                
					/* Perform FIR Band Pass filtering on ADC buffah   */
                    FIR(BLOCK_LENGTH,&FilterOut[0],&BufferA[0],&filterFilter);

					DAC1RDAT = BufferA[i];		// Load the DAC buffer with data
				}else{

					/* Perform FIR Band Pass filtering on ADC buffah   */
                    FIR(BLOCK_LENGTH,&FilterOut[0],&BufferB[0],&filterFilter);

					DAC1RDAT = BufferB[i];		// Load the DAC buffer with data
				}	
			}
			flag = 0;
		}		
	}
    
	return 0;

}

